# -*- coding: utf-8 -*-
# coding: utf-8
# emacs: -*- mode: python; py-indent-offset: 4; indent-tabs-mode: nil -*-
# vi: set ft=python sts=4 ts=4 sw=4 et:
"""Top-level namespace for elastix."""

from __future__ import absolute_import

from .registration import Registration, ApplyWarp, AnalyzeWarp, PointsWarp
from .utils import EditTransform



# AUTO-GENERATED by tools/checkspecs.py - DO NOT EDIT
from __future__ import unicode_literals
from ..specialized import BRAINSConstellationDetector


def test_BRAINSConstellationDetector_inputs():
    input_map = dict(
        BackgroundFillValue=dict(argstr='--BackgroundFillValue %s', ),
        LLSModel=dict(
            argstr='--LLSModel %s',
            extensions=None,
        ),
        acLowerBound=dict(argstr='--acLowerBound %f', ),
        args=dict(argstr='%s', ),
        atlasLandmarkWeights=dict(
            argstr='--atlasLandmarkWeights %s',
            extensions=None,
        ),
        atlasLandmarks=dict(
            argstr='--atlasLandmarks %s',
            extensions=None,
        ),
        atlasVolume=dict(
            argstr='--atlasVolume %s',
            extensions=None,
        ),
        cutOutHeadInOutputVolume=dict(argstr='--cutOutHeadInOutputVolume ', ),
        debug=dict(argstr='--debug ', ),
        environ=dict(
            nohash=True,
            usedefault=True,
        ),
        forceACPoint=dict(
            argstr='--forceACPoint %s',
            sep=',',
        ),
        forceHoughEyeDetectorReportFailure=dict(
            argstr='--forceHoughEyeDetectorReportFailure ', ),
        forcePCPoint=dict(
            argstr='--forcePCPoint %s',
            sep=',',
        ),
        forceRPPoint=dict(
            argstr='--forceRPPoint %s',
            sep=',',
        ),
        forceVN4Point=dict(
            argstr='--forceVN4Point %s',
            sep=',',
        ),
        houghEyeDetectorMode=dict(argstr='--houghEyeDetectorMode %d', ),
        inputLandmarksEMSP=dict(
            argstr='--inputLandmarksEMSP %s',
            extensions=None,
        ),
        inputTemplateModel=dict(
            argstr='--inputTemplateModel %s',
            extensions=None,
        ),
        inputVolume=dict(
            argstr='--inputVolume %s',
            extensions=None,
        ),
        interpolationMode=dict(argstr='--interpolationMode %s', ),
        mspQualityLevel=dict(argstr='--mspQualityLevel %d', ),
        numberOfThreads=dict(argstr='--numberOfThreads %d', ),
        otsuPercentileThreshold=dict(argstr='--otsuPercentileThreshold %f', ),
        outputLandmarksInACPCAlignedSpace=dict(
            argstr='--outputLandmarksInACPCAlignedSpace %s',
            hash_files=False,
        ),
        outputLandmarksInInputSpace=dict(
            argstr='--outputLandmarksInInputSpace %s',
            hash_files=False,
        ),
        outputMRML=dict(
            argstr='--outputMRML %s',
            hash_files=False,
        ),
        outputResampledVolume=dict(
            argstr='--outputResampledVolume %s',
            hash_files=False,
        ),
        outputTransform=dict(
            argstr='--outputTransform %s',
            hash_files=False,
        ),
        outputUntransformedClippedVolume=dict(
            argstr='--outputUntransformedClippedVolume %s',
            hash_files=False,
        ),
        outputVerificationScript=dict(
            argstr='--outputVerificationScript %s',
            hash_files=False,
        ),
        outputVolume=dict(
            argstr='--outputVolume %s',
            hash_files=False,
        ),
        rVN4=dict(argstr='--rVN4 %f', ),
        rac=dict(argstr='--rac %f', ),
        rescaleIntensities=dict(argstr='--rescaleIntensities ', ),
        rescaleIntensitiesOutputRange=dict(
            argstr='--rescaleIntensitiesOutputRange %s',
            sep=',',
        ),
        resultsDir=dict(
            argstr='--resultsDir %s',
            hash_files=False,
        ),
        rmpj=dict(argstr='--rmpj %f', ),
        rpc=dict(argstr='--rpc %f', ),
        trimRescaledIntensities=dict(argstr='--trimRescaledIntensities %f', ),
        verbose=dict(argstr='--verbose ', ),
        writeBranded2DImage=dict(
            argstr='--writeBranded2DImage %s',
            hash_files=False,
        ),
        writedebuggingImagesLevel=dict(
            argstr='--writedebuggingImagesLevel %d', ),
    )
    inputs = BRAINSConstellationDetector.input_spec()

    for key, metadata in list(input_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(inputs.traits()[key], metakey) == value
def test_BRAINSConstellationDetector_outputs():
    output_map = dict(
        outputLandmarksInACPCAlignedSpace=dict(extensions=None, ),
        outputLandmarksInInputSpace=dict(extensions=None, ),
        outputMRML=dict(extensions=None, ),
        outputResampledVolume=dict(extensions=None, ),
        outputTransform=dict(extensions=None, ),
        outputUntransformedClippedVolume=dict(extensions=None, ),
        outputVerificationScript=dict(extensions=None, ),
        outputVolume=dict(extensions=None, ),
        resultsDir=dict(),
        writeBranded2DImage=dict(extensions=None, ),
    )
    outputs = BRAINSConstellationDetector.output_spec()

    for key, metadata in list(output_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(outputs.traits()[key], metakey) == value



# -*- coding: utf-8 -*-
# emacs: -*- mode: python; py-indent-offset: 4; indent-tabs-mode: nil -*-
# vi: set ft=python sts=4 ts=4 sw=4 et:
from __future__ import print_function, unicode_literals
import os
import pytest
from builtins import open
from future import standard_library
from pkg_resources import resource_filename as pkgrf

from ....utils.filemanip import md5
from ... import base as nib

standard_library.install_aliases()


@pytest.mark.parametrize("args", [{}, {'a': 1, 'b': [2, 3]}])
def test_bunch(args):
    b = nib.Bunch(**args)
    assert b.__dict__ == args


def test_bunch_attribute():
    b = nib.Bunch(a=1, b=[2, 3], c=None)
    assert b.a == 1
    assert b.b == [2, 3]
    assert b.c is None


def test_bunch_repr():
    b = nib.Bunch(b=2, c=3, a=dict(n=1, m=2))
    assert repr(b) == "Bunch(a={'m': 2, 'n': 1}, b=2, c=3)"


def test_bunch_methods():
    b = nib.Bunch(a=2)
    b.update(a=3)
    newb = b.dictcopy()
    assert b.a == 3
    assert b.get('a') == 3
    assert b.get('badkey', 'otherthing') == 'otherthing'
    assert b != newb
    assert type(dict()) == type(newb)
    assert newb['a'] == 3


def test_bunch_hash():
    # NOTE: Since the path to the json file is included in the Bunch,
    # the hash will be unique to each machine.
    json_pth = pkgrf('nipype',
                     os.path.join('testing', 'data', 'realign_json.json'))

    b = nib.Bunch(infile=json_pth, otherthing='blue', yat=True)
    newbdict, bhash = b._get_bunch_hash()
    assert bhash == 'd1f46750044c3de102efc847720fc35f'
    # Make sure the hash stored in the json file for `infile` is correct.
    jshash = md5()
    with open(json_pth, 'r') as fp:
        jshash.update(fp.read().encode('utf-8'))
    assert newbdict['infile'][0][1] == jshash.hexdigest()
    assert newbdict['yat'] is True



# -*- coding: utf-8 -*-
import os
from ...base import Undefined
from ..model import Level1Design


def test_level1design(tmpdir):
    old = tmpdir.chdir()
    l = Level1Design()
    runinfo = dict(
        cond=[{
            'name': 'test_condition',
            'onset': [0, 10],
            'duration': [10, 10]
        }],
        regress=[])
    runidx = 0
    contrasts = Undefined
    do_tempfilter = False
    orthogonalization = {}
    basic_ev_parameters = {'temporalderiv': False}
    convolution_variants = [('custom', 7, {
        'temporalderiv': False,
        'bfcustompath': '/some/path'
    }), ('hrf', 3, basic_ev_parameters), ('dgamma', 3, basic_ev_parameters),
                            ('gamma', 2, basic_ev_parameters),
                            ('none', 0, basic_ev_parameters)]
    for key, val, ev_parameters in convolution_variants:
        output_num, output_txt = Level1Design._create_ev_files(
            l, os.getcwd(), runinfo, runidx, ev_parameters, orthogonalization,
            contrasts, do_tempfilter, key)
        assert "set fmri(convolve1) {0}".format(val) in output_txt



# AUTO-GENERATED by tools/checkspecs.py - DO NOT EDIT
from __future__ import unicode_literals
from ..minc import Norm


def test_Norm_inputs():
    input_map = dict(
        args=dict(argstr='%s', ),
        clamp=dict(
            argstr='-clamp',
            usedefault=True,
        ),
        clobber=dict(
            argstr='-clobber',
            usedefault=True,
        ),
        cutoff=dict(argstr='-cutoff %s', ),
        environ=dict(
            nohash=True,
            usedefault=True,
        ),
        input_file=dict(
            argstr='%s',
            extensions=None,
            mandatory=True,
            position=-2,
        ),
        lower=dict(argstr='-lower %s', ),
        mask=dict(
            argstr='-mask %s',
            extensions=None,
        ),
        out_ceil=dict(argstr='-out_ceil %s', ),
        out_floor=dict(argstr='-out_floor %s', ),
        output_file=dict(
            argstr='%s',
            extensions=None,
            genfile=True,
            hash_files=False,
            name_source=['input_file'],
            name_template='%s_norm.mnc',
            position=-1,
        ),
        output_threshold_mask=dict(
            argstr='-threshold_mask %s',
            extensions=None,
            hash_files=False,
            name_source=['input_file'],
            name_template='%s_norm_threshold_mask.mnc',
        ),
        threshold=dict(argstr='-threshold', ),
        threshold_blur=dict(argstr='-threshold_blur %s', ),
        threshold_bmt=dict(argstr='-threshold_bmt', ),
        threshold_perc=dict(argstr='-threshold_perc %s', ),
        upper=dict(argstr='-upper %s', ),
    )
    inputs = Norm.input_spec()

    for key, metadata in list(input_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(inputs.traits()[key], metakey) == value
def test_Norm_outputs():
    output_map = dict(
        output_file=dict(extensions=None, ),
        output_threshold_mask=dict(extensions=None, ),
    )
    outputs = Norm.output_spec()

    for key, metadata in list(output_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(outputs.traits()[key], metakey) == value



# -*- coding: utf-8 -*-
# -*- coding: utf8 -*-
"""Autogenerated file - DO NOT EDIT
If you spot a bug, please report it on the mailing list and/or change the generator."""

from nipype.interfaces.base import CommandLine, CommandLineInputSpec, SEMLikeCommandLine, TraitedSpec, File, Directory, traits, isdefined, InputMultiPath, OutputMultiPath
import os


class LandmarksCompareInputSpec(CommandLineInputSpec):
    inputLandmarkFile1 = File(
        desc="First input landmark file (.fcsv or .wts)",
        exists=True,
        argstr="--inputLandmarkFile1 %s")
    inputLandmarkFile2 = File(
        desc="Second input landmark file (.fcsv or .wts)",
        exists=True,
        argstr="--inputLandmarkFile2 %s")
    tolerance = traits.Float(
        desc=
        "The maximum error (in mm) allowed in each direction of a landmark",
        argstr="--tolerance %f")


class LandmarksCompareOutputSpec(TraitedSpec):
    pass


class LandmarksCompare(SEMLikeCommandLine):
    """title: Compare Fiducials

category: Testing

description: Compares two .fcsv or .wts text files and verifies that they are identicle.  Used for testing landmarks files.

contributor: Ali Ghayoor

"""

    input_spec = LandmarksCompareInputSpec
    output_spec = LandmarksCompareOutputSpec
    _cmd = " LandmarksCompare "
    _outputs_filenames = {}
    _redirect_x = False



# AUTO-GENERATED by tools/checkspecs.py - DO NOT EDIT
from __future__ import unicode_literals
from ..diffusion import dtiaverage


def test_dtiaverage_inputs():
    input_map = dict(
        DTI_double=dict(argstr='--DTI_double ', ),
        args=dict(argstr='%s', ),
        environ=dict(
            nohash=True,
            usedefault=True,
        ),
        inputs=dict(argstr='--inputs %s...', ),
        tensor_output=dict(
            argstr='--tensor_output %s',
            hash_files=False,
        ),
        verbose=dict(argstr='--verbose ', ),
    )
    inputs = dtiaverage.input_spec()

    for key, metadata in list(input_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(inputs.traits()[key], metakey) == value
def test_dtiaverage_outputs():
    output_map = dict(tensor_output=dict(extensions=None, ), )
    outputs = dtiaverage.output_spec()

    for key, metadata in list(output_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(outputs.traits()[key], metakey) == value



# AUTO-GENERATED by tools/checkspecs.py - DO NOT EDIT
from __future__ import unicode_literals
from ..developer import JistLaminarROIAveraging


def test_JistLaminarROIAveraging_inputs():
    input_map = dict(
        args=dict(argstr='%s', ),
        environ=dict(
            nohash=True,
            usedefault=True,
        ),
        inIntensity=dict(
            argstr='--inIntensity %s',
            extensions=None,
        ),
        inMask=dict(
            argstr='--inMask %s',
            extensions=None,
        ),
        inROI=dict(
            argstr='--inROI %s',
            extensions=None,
        ),
        inROI2=dict(argstr='--inROI2 %s', ),
        null=dict(argstr='--null %s', ),
        outROI3=dict(
            argstr='--outROI3 %s',
            hash_files=False,
        ),
        xDefaultMem=dict(argstr='-xDefaultMem %d', ),
        xMaxProcess=dict(
            argstr='-xMaxProcess %d',
            usedefault=True,
        ),
        xPrefExt=dict(argstr='--xPrefExt %s', ),
    )
    inputs = JistLaminarROIAveraging.input_spec()

    for key, metadata in list(input_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(inputs.traits()[key], metakey) == value
def test_JistLaminarROIAveraging_outputs():
    output_map = dict(outROI3=dict(extensions=None, ), )
    outputs = JistLaminarROIAveraging.output_spec()

    for key, metadata in list(output_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(outputs.traits()[key], metakey) == value



# AUTO-GENERATED by tools/checkspecs.py - DO NOT EDIT
from __future__ import unicode_literals
from ..brainsresample import BRAINSResample


def test_BRAINSResample_inputs():
    input_map = dict(
        args=dict(argstr='%s', ),
        defaultValue=dict(argstr='--defaultValue %f', ),
        deformationVolume=dict(
            argstr='--deformationVolume %s',
            extensions=None,
        ),
        environ=dict(
            nohash=True,
            usedefault=True,
        ),
        gridSpacing=dict(
            argstr='--gridSpacing %s',
            sep=',',
        ),
        inputVolume=dict(
            argstr='--inputVolume %s',
            extensions=None,
        ),
        interpolationMode=dict(argstr='--interpolationMode %s', ),
        inverseTransform=dict(argstr='--inverseTransform ', ),
        numberOfThreads=dict(argstr='--numberOfThreads %d', ),
        outputVolume=dict(
            argstr='--outputVolume %s',
            hash_files=False,
        ),
        pixelType=dict(argstr='--pixelType %s', ),
        referenceVolume=dict(
            argstr='--referenceVolume %s',
            extensions=None,
        ),
        warpTransform=dict(
            argstr='--warpTransform %s',
            extensions=None,
        ),
    )
    inputs = BRAINSResample.input_spec()

    for key, metadata in list(input_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(inputs.traits()[key], metakey) == value
def test_BRAINSResample_outputs():
    output_map = dict(outputVolume=dict(extensions=None, ), )
    outputs = BRAINSResample.output_spec()

    for key, metadata in list(output_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(outputs.traits()[key], metakey) == value



# emacs: -*- mode: python; py-indent-offset: 4; indent-tabs-mode: nil -*-
# vi: set ft=python sts=4 ts=4 sw=4 et:
"""
The dwi module of niftyfit, which wraps the fitting methods in NiftyFit.
"""

from ..base import File, TraitedSpec, traits, isdefined, CommandLineInputSpec
from .base import NiftyFitCommand
from ..niftyreg.base import get_custom_path


class FitDwiInputSpec(CommandLineInputSpec):
    """ Input Spec for FitDwi. """
    # Inputs options
    source_file = File(
        position=1,
        exists=True,
        argstr='-source %s',
        mandatory=True,
        desc='The source image containing the dwi data.')
    desc = 'The file containing the bvalues of the source DWI.'
    bval_file = File(
        position=2, exists=True, argstr='-bval %s', mandatory=True, desc=desc)
    desc = 'The file containing the bvectors of the source DWI.'
    bvec_file = File(
        position=3, exists=True, argstr='-bvec %s', mandatory=True, desc=desc)
    te_file = File(
        exists=True,
        argstr='-TE %s',
        desc='Filename of TEs (ms).',
        xor=['te_file'])
    te_value = File(
        exists=True,
        argstr='-TE %s',
        desc='Value of TEs (ms).',
        xor=['te_file'])
    mask_file = File(
        exists=True, desc='The image mask', argstr='-mask %s')
    desc = 'Filename of parameter priors for -ball and -nod.'
    prior_file = File(exists=True, argstr='-prior %s', desc=desc)
    desc = 'Rotate the output tensors according to the q/s form of the image \
(resulting tensors will be in mm coordinates, default: 0).'

    rot_sform_flag = traits.Int(desc=desc, argstr='-rotsform %d')

    # generic output options:
    error_file = File(
        name_source=['source_file'],
        name_template='%s_error.nii.gz',
        desc='Filename of parameter error maps.',
        argstr='-error %s')
    res_file = File(
        name_source=['source_file'],
        name_template='%s_resmap.nii.gz',
        desc='Filename of model residual map.',
        argstr='-res %s')
    syn_file = File(
        name_source=['source_file'],
        name_template='%s_syn.nii.gz',
        desc='Filename of synthetic image.',
        argstr='-syn %s')
    nodiff_file = File(
        name_source=['source_file'],
        name_template='%s_no_diff.nii.gz',
        desc='Filename of average no diffusion image.',
        argstr='-nodiff %s')

    # Output options, with templated output names based on the source image
    mcmap_file = File(
        name_source=['source_file'],
        name_template='%s_mcmap.nii.gz',
        desc='Filename of multi-compartment model parameter map '
             '(-ivim,-ball,-nod)',
        argstr='-mcmap %s',
        requires=['nodv_flag'])

    # Model Specific Output options:
    mdmap_file = File(
        name_source=['source_file'],
        name_template='%s_mdmap.nii.gz',
        desc='Filename of MD map/ADC',
        argstr='-mdmap %s')
    famap_file = File(
        name_source=['source_file'],
        name_template='%s_famap.nii.gz',
        desc='Filename of FA map',
        argstr='-famap %s')
    v1map_file = File(
        name_source=['source_file'],
        name_template='%s_v1map.nii.gz',
        desc='Filename of PDD map [x,y,z]',
        argstr='-v1map %s')
    rgbmap_file = File(
        name_source=['source_file'],
        name_template='%s_rgbmap.nii.gz',
        desc='Filename of colour-coded FA map',
        argstr='-rgbmap %s',
        requires=['dti_flag'])

    desc = 'Use lower triangular (tenmap2) or diagonal, off-diagonal tensor \
format'

    ten_type = traits.Enum(
        'lower-tri', 'diag-off-diag', desc=desc, usedefault=True)

    tenmap_file = File(
        name_source=['source_file'],
        name_template='%s_tenmap.nii.gz',
        desc='Filename of tensor map [diag,offdiag].',
        argstr='-tenmap %s',
        requires=['dti_flag'])
    tenmap2_file = File(
        name_source=['source_file'],
        name_template='%s_tenmap2.nii.gz',
        desc='Filename of tensor map [lower tri]',
        argstr='-tenmap2 %s',
        requires=['dti_flag'])

    # Methods options
    desc = 'Fit single exponential to non-directional data [default with \
no b-vectors]'

    mono_flag = traits.Bool(
        desc=desc,
        argstr='-mono',
        position=4,
        xor=[
            'ivim_flag', 'dti_flag', 'ball_flag', 'ballv_flag', 'nod_flag',
            'nodv_flag'
        ])
    ivim_flag = traits.Bool(
        desc='Fit IVIM model to non-directional data.',
        argstr='-ivim',
        position=4,
        xor=[
            'mono_flag', 'dti_flag', 'ball_flag', 'ballv_flag', 'nod_flag',
            'nodv_flag'
        ])
    desc = 'Fit the tensor model [default with b-vectors].'
    dti_flag = traits.Bool(
        desc=desc,
        argstr='-dti',
        position=4,
        xor=[
            'mono_flag', 'ivim_flag', 'ball_flag', 'ballv_flag', 'nod_flag',
            'nodv_flag'
        ])
    ball_flag = traits.Bool(
        desc='Fit the ball and stick model.',
        argstr='-ball',
        position=4,
        xor=[
            'mono_flag', 'ivim_flag', 'dti_flag', 'ballv_flag', 'nod_flag',
            'nodv_flag'
        ])
    desc = 'Fit the ball and stick model with optimised PDD.'
    ballv_flag = traits.Bool(
        desc=desc,
        argstr='-ballv',
        position=4,
        xor=[
            'mono_flag', 'ivim_flag', 'dti_flag', 'ball_flag', 'nod_flag',
            'nodv_flag'
        ])
    nod_flag = traits.Bool(
        desc='Fit the NODDI model',
        argstr='-nod',
        position=4,
        xor=[
            'mono_flag', 'ivim_flag', 'dti_flag', 'ball_flag', 'ballv_flag',
            'nodv_flag'
        ])
    nodv_flag = traits.Bool(
        desc='Fit the NODDI model with optimised PDD',
        argstr='-nodv',
        position=4,
        xor=[
            'mono_flag', 'ivim_flag', 'dti_flag', 'ball_flag', 'ballv_flag',
            'nod_flag'
        ])

    # Experimental options
    desc = 'Maximum number of non-linear LSQR iterations [100x2 passes])'
    maxit_val = traits.Int(desc=desc, argstr='-maxit %d', requires=['gn_flag'])
    desc = 'LM parameters (initial value, decrease rate) [100,1.2].'
    lm_vals = traits.Tuple(
        traits.Float,
        traits.Float,
        argstr='-lm %f %f',
        requires=['gn_flag'],
        desc=desc)
    desc = 'Use Gauss-Newton algorithm [Levenberg-Marquardt].'
    gn_flag = traits.Bool(desc=desc, argstr='-gn', xor=['wls_flag'])
    desc = 'Use Variational Bayes fitting with known prior (currently \
identity covariance...).'

    vb_flag = traits.Bool(desc=desc, argstr='-vb')
    cov_file = File(
        exists=True,
        desc='Filename of ithe nc*nc covariance matrix [I]',
        argstr='-cov %s')
    wls_flag = traits.Bool(desc=desc, argstr='-wls', xor=['gn_flag'])
    desc = 'Use location-weighted least squares for DTI fitting [3x3 Gaussian]'
    swls_val = traits.Float(desc=desc, argstr='-swls %f')
    slice_no = traits.Int(
        desc='Fit to single slice number.', argstr='-slice %d')
    voxel = traits.Tuple(
        traits.Int,
        traits.Int,
        traits.Int,
        desc='Fit to single voxel only.',
        argstr='-voxel %d %d %d')
    diso_val = traits.Float(
        desc='Isotropic diffusivity for -nod [3e-3]', argstr='-diso %f')
    dpr_val = traits.Float(
        desc='Parallel diffusivity for -nod [1.7e-3].', argstr='-dpr %f')
    wm_t2_val = traits.Float(
        desc='White matter T2 value [80ms].', argstr='-wmT2 %f')
    csf_t2_val = traits.Float(desc='CSF T2 value [400ms].', argstr='-csfT2 %f')
    desc = 'Threshold for perfusion/diffsuion effects [100].'
    perf_thr = traits.Float(desc=desc, argstr='-perfthreshold %f')

    # MCMC options:
    mcout = File(
        name_source=['source_file'],
        name_template='%s_mcout.txt',
        desc='Filename of mc samples (ascii text file)',
        argstr='-mcout %s')
    mcsamples = traits.Int(
        desc='Number of samples to keep [100].', argstr='-mcsamples %d')
    mcmaxit = traits.Int(
        desc='Number of iterations to run [10,000].', argstr='-mcmaxit %d')
    acceptance = traits.Float(
        desc='Fraction of iterations to accept [0.23].',
        argstr='-accpetance %f')


class FitDwiOutputSpec(TraitedSpec):
    """ Output Spec for FitDwi. """

    error_file = File(desc='Filename of parameter error maps')
    res_file = File(desc='Filename of model residual map')
    syn_file = File(desc='Filename of synthetic image')
    nodiff_file = File(desc='Filename of average no diffusion image.')
    mdmap_file = File(desc='Filename of MD map/ADC')
    famap_file = File(desc='Filename of FA map')
    v1map_file = File(desc='Filename of PDD map [x,y,z]')
    rgbmap_file = File(desc='Filename of colour FA map')
    tenmap_file = File(desc='Filename of tensor map')
    tenmap2_file = File(desc='Filename of tensor map [lower tri]')

    mcmap_file = File(desc='Filename of multi-compartment model '
                                  'parameter map (-ivim,-ball,-nod).')
    mcout = File(desc='Filename of mc samples (ascii text file)')


class FitDwi(NiftyFitCommand):
    """Interface for executable fit_dwi from Niftyfit platform.

    Use NiftyFit to perform diffusion model fitting.

    Diffusion-weighted MR Fitting.
    Fits DWI parameter maps to multi-shell, multi-directional data.

    `Source code <https://cmiclab.cs.ucl.ac.uk/CMIC/NiftyFit-Release>`_

    Examples
    --------

    >>> from nipype.interfaces import niftyfit
    >>> fit_dwi = niftyfit.FitDwi(dti_flag=True)
    >>> fit_dwi.inputs.source_file = 'dwi.nii.gz'
    >>> fit_dwi.inputs.bvec_file = 'bvecs'
    >>> fit_dwi.inputs.bval_file = 'bvals'
    >>> fit_dwi.inputs.rgbmap_file = 'rgb.nii.gz'
    >>> fit_dwi.cmdline
    'fit_dwi -source dwi.nii.gz -bval bvals -bvec bvecs -dti \
-error dwi_error.nii.gz -famap dwi_famap.nii.gz -mcout dwi_mcout.txt \
-mdmap dwi_mdmap.nii.gz -nodiff dwi_no_diff.nii.gz -res dwi_resmap.nii.gz \
-rgbmap rgb.nii.gz -syn dwi_syn.nii.gz -tenmap2 dwi_tenmap2.nii.gz \
-v1map dwi_v1map.nii.gz'

    """
    _cmd = get_custom_path('fit_dwi', env_dir='NIFTYFITDIR')
    input_spec = FitDwiInputSpec
    output_spec = FitDwiOutputSpec
    _suffix = '_fit_dwi'

    def _format_arg(self, name, trait_spec, value):
        if name == 'tenmap_file' and self.inputs.ten_type != 'diag-off-diag':
            return ''
        if name == 'tenmap2_file' and self.inputs.ten_type != 'lower-tri':
            return ''
        return super(FitDwi, self)._format_arg(name, trait_spec, value)


class DwiToolInputSpec(CommandLineInputSpec):
    """ Input Spec for DwiTool. """
    desc = 'The source image containing the fitted model.'
    source_file = File(
        position=1,
        exists=True,
        desc=desc,
        argstr='-source %s',
        mandatory=True)
    desc = 'The file containing the bvalues of the source DWI.'
    bval_file = File(
        position=2, exists=True, desc=desc, argstr='-bval %s', mandatory=True)
    desc = 'The file containing the bvectors of the source DWI.'
    bvec_file = File(
        position=3, exists=True, desc=desc, argstr='-bvec %s')
    b0_file = File(
        position=4,
        exists=True,
        desc='The B0 image corresponding to the source DWI',
        argstr='-b0 %s')
    mask_file = File(
        position=5, exists=True, desc='The image mask', argstr='-mask %s')

    # Output options, with templated output names based on the source image
    desc = 'Filename of multi-compartment model parameter map \
(-ivim,-ball,-nod)'

    mcmap_file = File(
        name_source=['source_file'],
        name_template='%s_mcmap.nii.gz',
        desc=desc,
        argstr='-mcmap %s')
    desc = 'Filename of synthetic image. Requires: bvec_file/b0_file.'
    syn_file = File(
        name_source=['source_file'],
        name_template='%s_syn.nii.gz',
        desc=desc,
        argstr='-syn %s',
        requires=['bvec_file', 'b0_file'])
    mdmap_file = File(
        name_source=['source_file'],
        name_template='%s_mdmap.nii.gz',
        desc='Filename of MD map/ADC',
        argstr='-mdmap %s')
    famap_file = File(
        name_source=['source_file'],
        name_template='%s_famap.nii.gz',
        desc='Filename of FA map',
        argstr='-famap %s')
    v1map_file = File(
        name_source=['source_file'],
        name_template='%s_v1map.nii.gz',
        desc='Filename of PDD map [x,y,z]',
        argstr='-v1map %s')
    rgbmap_file = File(
        name_source=['source_file'],
        name_template='%s_rgbmap.nii.gz',
        desc='Filename of colour FA map.',
        argstr='-rgbmap %s')
    logdti_file = File(
        name_source=['source_file'],
        name_template='%s_logdti2.nii.gz',
        desc='Filename of output logdti map.',
        argstr='-logdti2 %s')

    # Methods options
    desc = 'Input is a single exponential to non-directional data \
[default with no b-vectors]'

    mono_flag = traits.Bool(
        desc=desc,
        position=6,
        argstr='-mono',
        xor=[
            'ivim_flag', 'dti_flag', 'dti_flag2', 'ball_flag', 'ballv_flag',
            'nod_flag', 'nodv_flag'
        ])
    desc = 'Inputs is an IVIM model to non-directional data.'
    ivim_flag = traits.Bool(
        desc=desc,
        position=6,
        argstr='-ivim',
        xor=[
            'mono_flag', 'dti_flag', 'dti_flag2', 'ball_flag', 'ballv_flag',
            'nod_flag', 'nodv_flag'
        ])
    dti_flag = traits.Bool(
        desc='Input is a tensor model diag/off-diag.',
        position=6,
        argstr='-dti',
        xor=[
            'mono_flag', 'ivim_flag', 'dti_flag2', 'ball_flag', 'ballv_flag',
            'nod_flag', 'nodv_flag'
        ])
    dti_flag2 = traits.Bool(
        desc='Input is a tensor model lower triangular',
        position=6,
        argstr='-dti2',
        xor=[
            'mono_flag', 'ivim_flag', 'dti_flag', 'ball_flag', 'ballv_flag',
            'nod_flag', 'nodv_flag'
        ])
    ball_flag = traits.Bool(
        desc='Input is a ball and stick model.',
        position=6,
        argstr='-ball',
        xor=[
            'mono_flag', 'ivim_flag', 'dti_flag', 'dti_flag2', 'ballv_flag',
            'nod_flag', 'nodv_flag'
        ])
    desc = 'Input is a ball and stick model with optimised PDD.'
    ballv_flag = traits.Bool(
        desc=desc,
        position=6,
        argstr='-ballv',
        xor=[
            'mono_flag', 'ivim_flag', 'dti_flag', 'dti_flag2', 'ball_flag',
            'nod_flag', 'nodv_flag'
        ])
    nod_flag = traits.Bool(
        desc='Input is a NODDI model',
        position=6,
        argstr='-nod',
        xor=[
            'mono_flag', 'ivim_flag', 'dti_flag', 'dti_flag2', 'ball_flag',
            'ballv_flag', 'nodv_flag'
        ])
    nodv_flag = traits.Bool(
        desc='Input is a NODDI model with optimised PDD',
        position=6,
        argstr='-nodv',
        xor=[
            'mono_flag', 'ivim_flag', 'dti_flag', 'dti_flag2', 'ball_flag',
            'ballv_flag', 'nod_flag'
        ])

    # Experimental options
    diso_val = traits.Float(
        desc='Isotropic diffusivity for -nod [3e-3]', argstr='-diso %f')
    dpr_val = traits.Float(
        desc='Parallel diffusivity for -nod [1.7e-3].', argstr='-dpr %f')


class DwiToolOutputSpec(TraitedSpec):
    """ Output Spec for DwiTool. """
    desc = 'Filename of multi-compartment model parameter map \
(-ivim,-ball,-nod)'

    mcmap_file = File(desc=desc)
    syn_file = File(desc='Filename of synthetic image')
    mdmap_file = File(desc='Filename of MD map/ADC')
    famap_file = File(desc='Filename of FA map')
    v1map_file = File(desc='Filename of PDD map [x,y,z]')
    rgbmap_file = File(desc='Filename of colour FA map')
    logdti_file = File(desc='Filename of output logdti map')


class DwiTool(NiftyFitCommand):
    """Interface for executable dwi_tool from Niftyfit platform.

    Use DwiTool.

    Diffusion-Weighted MR Prediction.
    Predicts DWI from previously fitted models and calculates model derived
    maps.

    `Source code <https://cmiclab.cs.ucl.ac.uk/CMIC/NiftyFit-Release>`_

    Examples
    --------

    >>> from nipype.interfaces import niftyfit
    >>> dwi_tool = niftyfit.DwiTool(dti_flag=True)
    >>> dwi_tool.inputs.source_file = 'dwi.nii.gz'
    >>> dwi_tool.inputs.bvec_file = 'bvecs'
    >>> dwi_tool.inputs.bval_file = 'bvals'
    >>> dwi_tool.inputs.mask_file = 'mask.nii.gz'
    >>> dwi_tool.inputs.b0_file = 'b0.nii.gz'
    >>> dwi_tool.inputs.rgbmap_file = 'rgb_map.nii.gz'
    >>> dwi_tool.cmdline
    'dwi_tool -source dwi.nii.gz -bval bvals -bvec bvecs -b0 b0.nii.gz \
-mask mask.nii.gz -dti -famap dwi_famap.nii.gz -logdti2 dwi_logdti2.nii.gz \
-mcmap dwi_mcmap.nii.gz -mdmap dwi_mdmap.nii.gz -rgbmap rgb_map.nii.gz \
-syn dwi_syn.nii.gz -v1map dwi_v1map.nii.gz'

    """
    _cmd = get_custom_path('dwi_tool', env_dir='NIFTYFITDIR')
    input_spec = DwiToolInputSpec
    output_spec = DwiToolOutputSpec
    _suffix = '_dwi_tool'

    def _format_arg(self, name, trait_spec, value):
        if name == 'syn_file':
            if not isdefined(self.inputs.bvec_file) or \
               not isdefined(self.inputs.b0_file):
                return ''
        if name in ['logdti_file', 'rgbmap_file']:
            if not isdefined(self.inputs.dti_flag) and \
               not isdefined(self.inputs.dti_flag2):
                return ''
        return super(DwiTool, self)._format_arg(name, trait_spec, value)



# AUTO-GENERATED by tools/checkspecs.py - DO NOT EDIT
from __future__ import unicode_literals
from ..reconstruction import CSD


def test_CSD_inputs():
    input_map = dict(
        b0_thres=dict(usedefault=True, ),
        in_bval=dict(
            extensions=None,
            mandatory=True,
        ),
        in_bvec=dict(
            extensions=None,
            mandatory=True,
        ),
        in_file=dict(
            extensions=None,
            mandatory=True,
        ),
        in_mask=dict(extensions=None, ),
        out_fods=dict(extensions=None, ),
        out_prefix=dict(),
        response=dict(extensions=None, ),
        save_fods=dict(usedefault=True, ),
        sh_order=dict(usedefault=True, ),
    )
    inputs = CSD.input_spec()

    for key, metadata in list(input_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(inputs.traits()[key], metakey) == value
def test_CSD_outputs():
    output_map = dict(
        model=dict(extensions=None, ),
        out_fods=dict(extensions=None, ),
    )
    outputs = CSD.output_spec()

    for key, metadata in list(output_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(outputs.traits()[key], metakey) == value



# AUTO-GENERATED by tools/checkspecs.py - DO NOT EDIT
from __future__ import unicode_literals
from ..preprocess import Retroicor


def test_Retroicor_inputs():
    input_map = dict(
        args=dict(argstr='%s', ),
        card=dict(
            argstr='-card %s',
            extensions=None,
            position=-2,
        ),
        cardphase=dict(
            argstr='-cardphase %s',
            extensions=None,
            hash_files=False,
            position=-6,
        ),
        environ=dict(
            nohash=True,
            usedefault=True,
        ),
        in_file=dict(
            argstr='%s',
            copyfile=False,
            extensions=None,
            mandatory=True,
            position=-1,
        ),
        num_threads=dict(
            nohash=True,
            usedefault=True,
        ),
        order=dict(
            argstr='-order %s',
            position=-5,
        ),
        out_file=dict(
            argstr='-prefix %s',
            extensions=None,
            name_source=['in_file'],
            name_template='%s_retroicor',
            position=1,
        ),
        outputtype=dict(),
        resp=dict(
            argstr='-resp %s',
            extensions=None,
            position=-3,
        ),
        respphase=dict(
            argstr='-respphase %s',
            extensions=None,
            hash_files=False,
            position=-7,
        ),
        threshold=dict(
            argstr='-threshold %d',
            position=-4,
        ),
    )
    inputs = Retroicor.input_spec()

    for key, metadata in list(input_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(inputs.traits()[key], metakey) == value
def test_Retroicor_outputs():
    output_map = dict(out_file=dict(extensions=None, ), )
    outputs = Retroicor.output_spec()

    for key, metadata in list(output_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(outputs.traits()[key], metakey) == value



# AUTO-GENERATED by tools/checkspecs.py - DO NOT EDIT
from __future__ import unicode_literals
from ..utils import WarpPoints


def test_WarpPoints_inputs():
    input_map = dict(
        args=dict(argstr='%s', ),
        coord_mm=dict(
            argstr='-mm',
            xor=['coord_vox'],
        ),
        coord_vox=dict(
            argstr='-vox',
            xor=['coord_mm'],
        ),
        dest_file=dict(
            argstr='-dest %s',
            extensions=None,
            mandatory=True,
        ),
        environ=dict(
            nohash=True,
            usedefault=True,
        ),
        in_coords=dict(
            argstr='%s',
            extensions=None,
            mandatory=True,
            position=-1,
        ),
        out_file=dict(
            extensions=None,
            name_source='in_coords',
            name_template='%s_warped',
            output_name='out_file',
        ),
        src_file=dict(
            argstr='-src %s',
            extensions=None,
            mandatory=True,
        ),
        warp_file=dict(
            argstr='-warp %s',
            extensions=None,
            xor=['xfm_file'],
        ),
        xfm_file=dict(
            argstr='-xfm %s',
            extensions=None,
            xor=['warp_file'],
        ),
    )
    inputs = WarpPoints.input_spec()

    for key, metadata in list(input_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(inputs.traits()[key], metakey) == value
def test_WarpPoints_outputs():
    output_map = dict(out_file=dict(extensions=None, ), )
    outputs = WarpPoints.output_spec()

    for key, metadata in list(output_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(outputs.traits()[key], metakey) == value
